SystemOrganization addCategory: #'Seaside-FileSystem'!

!WAPackage class methodsFor: '*seaside-filesystem' stamp: 'jf 1/22/2009 03:24'!
seasideFileSystem
	^ self new
		name: 'Seaside-FileSystem';
		addDependency: 'Seaside-Core';
		yourself! !

WAEntryPoint subclass: #WAExternalFileLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-FileSystem'!

!WAExternalFileLibrary class methodsFor: 'accessing' stamp: 'lr 2/10/2008 14:49'!
description
	^ 'File Directory'! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 10/18/2008 11:26'!
asAbsoluteUrl: aFileName relativeTo: aRequest
	^aRequest url copy
		addToPath: aFileName;
		yourself! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 4/20/2008 21:18'!
contentsOfFile: aFilename
	| stream |
	stream := aFilename readStream.
	^[
		stream binary.
		stream upToEnd ]
		ensure: [ stream close ]! !

!WAExternalFileLibrary methodsFor: 'configuration' stamp: 'pmm 10/18/2008 10:56'!
defaultConfiguration
	^ super defaultConfiguration
		addParent: WAExternalFileLibraryConfiguration instance;
		yourself! !

!WAExternalFileLibrary methodsFor: 'accessing' stamp: 'pmm 10/18/2008 11:10'!
directory
	| path filename |
	path := self preferenceAt: #directory.
	filename := SpFilename named: path.
	"hack around SPort bugs"
	^(path last = filename separator)
		ifTrue: [ SpFilename named: path allButLast ]
		ifFalse: [ filename ]! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 10/18/2008 11:33'!
filenameForRequest: aRequest
	| filename basePathUrl |
	filename := self directory.
	basePathUrl := WAUrl new parsePath: self basePath; yourself.
	(basePathUrl relativeTo: aRequest url) do: [ :each |
		filename := filename construct: each ].
	^filename! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'jf 1/22/2009 02:43'!
filesIn: aPath do: aOneArgumentBlock
	"this can not be done with SPort"
	(WAPlatform current filesIn: aPath) do: [ :each |
		aOneArgumentBlock value: (WAPlatform current localNameOf: each) ]! !

!WAExternalFileLibrary methodsFor: 'handling' stamp: 'lr 2/3/2009 20:57'!
handleFiltered: aRequestContext
	[ self processContext: aRequestContext ]
		on: Error 
		do: [ :error | 
			aRequestContext responseFactory 
				internalError: error; 
				respond ]! !

!WAExternalFileLibrary methodsFor: 'accessing' stamp: 'pmm 10/18/2008 11:10'!
listing
	^ self preferenceAt: #listing! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 10/18/2008 11:30'!
mimetypeForRequest: aRequest
	| path |
	path := aRequest url path.
	^(path isEmpty or: [ (path last includes: $.) not ])
		ifTrue: [ WAFileLibrary defaultMimeType seasideMimeType ]
		ifFalse: [
			| extension |
			extension := path last copyAfterLast: $..
			WAFileLibrary mimetypeFor: extension ]! !

!WAExternalFileLibrary methodsFor: 'handling' stamp: 'lr 2/3/2009 21:00'!
processContext: aRequestContext
	| filename |
	filename := self filenameForRequest: aRequestContext request.
	(self listing and: [ filename isDirectory ])
		ifTrue: [ self processDirectory: filename asString context: aRequestContext ]
		ifFalse: [ self processFile: filename context: aRequestContext ]! !

!WAExternalFileLibrary methodsFor: 'handling' stamp: 'jf 2/15/2009 21:24'!
processDirectory: aString context: aRequestContext
	| contents |
	contents := WARenderCanvas builder
		fullDocument: true;
		rootBlock: [ :html | html title: aRequestContext request url seasideString ];
		render: [ :html |
			html heading
				level: 1;
				with: aRequestContext request url.
			html unorderedList: [
				self filesIn: aString do: [ :each | 
					html listItem: [
						html anchor
							url: (self asAbsoluteUrl: each relativeTo: aRequestContext request);
							with: each ] ] ] ].
	aRequestContext response
		contentType: WAMimeType textHtml;
		nextPutAll: contents;
		respond! !

!WAExternalFileLibrary methodsFor: 'handling' stamp: 'lr 2/3/2009 20:53'!
processFile: aFilename context: aRequestContext
	aRequestContext response
		document: (self contentsOfFile: aFilename)
		mimeType: (self mimetypeForRequest: aRequestContext request)
		fileName: (aRequestContext request url path last);
		respond! !

WASystemConfiguration subclass: #WAExternalFileLibraryConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-FileSystem'!

!WAExternalFileLibraryConfiguration methodsFor: 'attributes' stamp: 'pmm 10/18/2008 11:12'!
describeOn: config
	(config string: #directory)
		label: 'Directory';
		comment: 'The base directory to serve files from.';
		default: (SpFilename named: 'xxxDoesNotExists') asAbsoluteFilename directory asString.
	(config boolean: #listing)
		label: 'Allow Listing';
		comment: 'Enable listing of directory entries.';
		default: false! !
